/*
 *	OSS style mixer stub program. (C) 1998 Donald M Burns.
 *
 *	Version		Description				Date
 *	0	Well, you've got to start somewhere.		6 Jan 1998
 *
 *	1	Redundant ioctl deleted.			16 Jan 1998
 */
 
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <errno.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/soundcard.h>

char *mixer = "/dev/mixer";
static char *pname;
static char *dname[] = SOUND_DEVICE_NAMES;
static char *label[] = SOUND_DEVICE_LABELS;
static char *type[] = { "mono", "stereo" };
static char *rec[] = { "normal", "source" };

/* write a "goodbye world" note and top yourself */
static void suicide(char *str)
{
	if (!errno)
		fprintf(stderr, "%s: %s\n", pname, str);
	else
		fprintf(stderr, "%s: \"%s\" %s\n", pname, str, strerror(errno));

	exit(1);
}

/* read the mixer and report to the stdio */
static void report(char *mname)
{
	int mfd, devno, dmask, rmask, smask, vol;

	if ((mfd = open(mname, O_RDWR)) < 0) suicide(mname);
	
	if (ioctl(mfd, SOUND_MIXER_READ_DEVMASK, &dmask) == -1)
		suicide("SOUND_MIXER_READ_DEVMASK");
	if (ioctl(mfd, SOUND_MIXER_READ_RECMASK, &rmask) == -1)
		suicide("SOUND_MIXER_READ_RECMASK");
	if (ioctl(mfd, SOUND_MIXER_READ_STEREODEVS, &smask) == -1)
		suicide("SOUND_MIXER_READ_STEREODEVS");

	for (devno = 0; devno < SOUND_MIXER_NRDEVICES; ++devno)
		if ((dmask>>devno)&1 || (rmask>>devno)&1) { /* do we have this device? */
			if ((dmask>>devno)&1) {
				if (ioctl(mfd, MIXER_READ(devno), &vol) < 0)
					suicide("MIXER_READ"); 
				printf("%s:%s:%d:%d:%s:%s\n",
					dname[devno],
					label[devno],
					vol&0xFF, (vol>>8)&0xFF,
					rec[(rmask>>devno)&1],
					type[(smask>>devno)&1] );
			} else /* or is just a record source ? */
				printf("%s:%s:%d:%d:%s:%s\n",
					dname[devno],
					label[devno],
					-1, -1,
					rec[(rmask>>devno)&1],
					"none");
		}
}

static void mixset(char *mname, int ac, char *av[])
{
	int mfd, devno, vol, rec, dmask;

	if (ac != 4) suicide("wrong number of arguments");
	
	if ((mfd = open(mname, O_RDWR)) < 0) suicide(mname);
	
	for (devno = 0; devno < SOUND_MIXER_NRDEVICES; ++devno)
		if (!strcmp(av[0],dname[devno])) break;

	if (devno == SOUND_MIXER_NRDEVICES)
		suicide("bad mixer channel name");

	if (ioctl(mfd, SOUND_MIXER_READ_DEVMASK, &dmask) == -1)
		suicide("SOUND_MIXER_READ_DEVMASK");

	/* only update the level if this device has that feature */		
	if ((dmask>>devno)&1) {

		vol = atoi(av[1])&0xFF | (atoi(av[2])&0xFF)<<8;

		if (ioctl(mfd, MIXER_WRITE(devno), &vol) < 0)
			suicide("MIXER_WRITE");
	}
	
	if (ioctl(mfd, SOUND_MIXER_READ_RECSRC, &rec) < 0)
		suicide("SOUND_MIXER_READ_RECSRC");

	if (!strcmp(av[3],"record"))
		rec |= 1<<devno;	/* set the device bit */
	else if (!strcmp(av[3],"normal"))
		rec &= ~(1<<devno);	/* clear the device bit */
	else suicide("bad record parameter name");

	/* set the recording source(s):- rec is written with the result */
	if (ioctl(mfd, SOUND_MIXER_WRITE_RECSRC, &rec) < 0)
		suicide("MIXER_WRITE_RECSRC");

	/* report back which channels are ACTUALLY being recorded */
	for (devno = 0; devno < SOUND_MIXER_NRDEVICES; ++devno)
		if ((rec>>devno)&1) /* is the channel recording? */
			printf("%s\n",dname[devno]);
}

int main(int argc, char *argv[])
{
	pname = argv[0];
	++argv;
	--argc;

	/* if the first arg is a path, then that's the mixer */
	if ((argc >= 1) && (**argv == '/')) {
		mixer = *argv++;
		--argc;
	}

	/* if there is no none-path args then report the status */
	if (!argc)
		report(mixer);
 	else
 		mixset(mixer, argc, argv);

 	return 0;
}
